Struct RopeInfo
pub struct RopeInfo { /* private fields */ }Trait Implementations
impl DefaultMetric for RopeInfo
impl DefaultMetric for RopeInfo
type DefaultMetric = BaseMetric
impl Diff<RopeInfo> for LineHashDiff
impl Diff<RopeInfo> for LineHashDiff
impl Metric<RopeInfo> for BaseMetric
impl Metric<RopeInfo> for BaseMetric
fn measure(_: &RopeInfo, len: usize) -> usize
fn measure(_: &RopeInfo, len: usize) -> usize
Return the size of the
NodeInfo::L, as measured by this
metric. Read more
fn to_base_units(s: &String, in_measured_units: usize) -> usize
fn to_base_units(s: &String, in_measured_units: usize) -> usize
Returns the smallest offset, in base units, for an offset in measured units. Read more
fn from_base_units(s: &String, in_base_units: usize) -> usize
fn from_base_units(s: &String, in_base_units: usize) -> usize
Returns the smallest offset in measured units corresponding to an offset in base units. Read more
fn is_boundary(s: &String, offset: usize) -> bool
fn is_boundary(s: &String, offset: usize) -> bool
Return whether the offset in base units is a boundary of this metric.
If a boundary is at end of a leaf then this method must return true.
However, a boundary at the beginning of a leaf is optional
(the previous leaf will be queried).
fn prev(s: &String, offset: usize) -> Option<usize>
fn prev(s: &String, offset: usize) -> Option<usize>
Returns the index of the boundary directly preceding offset,
or None if no such boundary exists. Input and result are in base units.
fn next(s: &String, offset: usize) -> Option<usize>
fn next(s: &String, offset: usize) -> Option<usize>
Returns the index of the first boundary for which index > offset,
or None if no such boundary exists. Input and result are in base units.
fn can_fragment() -> bool
fn can_fragment() -> bool
Returns true if the measured units in this metric can span multiple
leaves. As an example, in a metric that measures lines in a rope, a
line may start in one leaf and end in another; however in a metric
measuring bytes, storage of a single byte cannot extend across leaves.
impl Metric<RopeInfo> for LinesMetric
Measured unit is newline amount.
Base unit is utf8 code unit.
Boundary is trailing and determined by a newline char.
impl Metric<RopeInfo> for LinesMetric
Measured unit is newline amount. Base unit is utf8 code unit. Boundary is trailing and determined by a newline char.
fn measure(info: &RopeInfo, _: usize) -> usize
fn measure(info: &RopeInfo, _: usize) -> usize
Return the size of the
NodeInfo::L, as measured by this
metric. Read more
fn is_boundary(s: &String, offset: usize) -> bool
fn is_boundary(s: &String, offset: usize) -> bool
Return whether the offset in base units is a boundary of this metric.
If a boundary is at end of a leaf then this method must return true.
However, a boundary at the beginning of a leaf is optional
(the previous leaf will be queried).
fn to_base_units(s: &String, in_measured_units: usize) -> usize
fn to_base_units(s: &String, in_measured_units: usize) -> usize
Returns the smallest offset, in base units, for an offset in measured units. Read more
fn from_base_units(s: &String, in_base_units: usize) -> usize
fn from_base_units(s: &String, in_base_units: usize) -> usize
Returns the smallest offset in measured units corresponding to an offset in base units. Read more
fn prev(s: &String, offset: usize) -> Option<usize>
fn prev(s: &String, offset: usize) -> Option<usize>
Returns the index of the boundary directly preceding offset,
or None if no such boundary exists. Input and result are in base units.
fn next(s: &String, offset: usize) -> Option<usize>
fn next(s: &String, offset: usize) -> Option<usize>
Returns the index of the first boundary for which index > offset,
or None if no such boundary exists. Input and result are in base units.
fn can_fragment() -> bool
fn can_fragment() -> bool
Returns true if the measured units in this metric can span multiple
leaves. As an example, in a metric that measures lines in a rope, a
line may start in one leaf and end in another; however in a metric
measuring bytes, storage of a single byte cannot extend across leaves.
impl Metric<RopeInfo> for Utf16CodeUnitsMetric
impl Metric<RopeInfo> for Utf16CodeUnitsMetric
fn measure(info: &RopeInfo, _: usize) -> usize
fn measure(info: &RopeInfo, _: usize) -> usize
Return the size of the
NodeInfo::L, as measured by this
metric. Read more
fn is_boundary(s: &String, offset: usize) -> bool
fn is_boundary(s: &String, offset: usize) -> bool
Return whether the offset in base units is a boundary of this metric.
If a boundary is at end of a leaf then this method must return true.
However, a boundary at the beginning of a leaf is optional
(the previous leaf will be queried).
fn to_base_units(s: &String, in_measured_units: usize) -> usize
fn to_base_units(s: &String, in_measured_units: usize) -> usize
Returns the smallest offset, in base units, for an offset in measured units. Read more
fn from_base_units(s: &String, in_base_units: usize) -> usize
fn from_base_units(s: &String, in_base_units: usize) -> usize
Returns the smallest offset in measured units corresponding to an offset in base units. Read more
fn prev(s: &String, offset: usize) -> Option<usize>
fn prev(s: &String, offset: usize) -> Option<usize>
Returns the index of the boundary directly preceding offset,
or None if no such boundary exists. Input and result are in base units.
fn next(s: &String, offset: usize) -> Option<usize>
fn next(s: &String, offset: usize) -> Option<usize>
Returns the index of the first boundary for which index > offset,
or None if no such boundary exists. Input and result are in base units.
fn can_fragment() -> bool
fn can_fragment() -> bool
Returns true if the measured units in this metric can span multiple
leaves. As an example, in a metric that measures lines in a rope, a
line may start in one leaf and end in another; however in a metric
measuring bytes, storage of a single byte cannot extend across leaves.
impl NodeInfo for RopeInfo
impl NodeInfo for RopeInfo
fn accumulate(&mut self, other: &RopeInfo)
fn accumulate(&mut self, other: &RopeInfo)
An operator that combines info from two subtrees. It is intended
(but not strictly enforced) that this operator be associative and
obey an identity property. In mathematical terms, the accumulate
method is the operation of a monoid.
fn compute_info(s: &String) -> RopeInfo
fn compute_info(s: &String) -> RopeInfo
A mapping from a leaf into the info type. It is intended (but
not strictly enforced) that applying the accumulate method to
the info derived from two leaves gives the same result as
deriving the info from the concatenation of the two leaves. In
mathematical terms, the compute_info method is a monoid
homomorphism.
impl Copy for RopeInfo