Struct Transformer
pub struct Transformer<'a, N>where
N: NodeInfo + 'a,{ /* private fields */ }Expand description
A mapping from coordinates in the source sequence to coordinates in the sequence after the delta is applied.
Implementations
impl<'a, N> Transformer<'a, N>where
N: NodeInfo + 'a,
impl<'a, N> Transformer<'a, N>where
N: NodeInfo + 'a,
pub fn new(delta: &'a Delta<N>) -> Transformer<'a, N>
pub fn new(delta: &'a Delta<N>) -> Transformer<'a, N>
Create a new transformer from a delta.
pub fn transform(&mut self, ix: usize, after: bool) -> usize
pub fn transform(&mut self, ix: usize, after: bool) -> usize
Transform a single coordinate. The after parameter indicates whether it
it should land before or after an inserted region.
pub fn interval_untouched<T>(&mut self, iv: T) -> boolwhere
T: IntervalBounds,
pub fn interval_untouched<T>(&mut self, iv: T) -> boolwhere
T: IntervalBounds,
Determine whether a given interval is untouched by the transformation.