Struct Cursor
pub struct Cursor<'a, N>where
N: 'a + NodeInfo,{ /* private fields */ }Expand description
A data structure for traversing boundaries in a tree.
It is designed to be efficient both for random access and for iteration. The
cursor itself is agnostic to which Metric is used to determine boundaries, but
the methods to find boundaries are parametrized on the Metric.
A cursor can be valid or invalid. It is always valid when created or after
set is called, and becomes invalid after prev
or next fails to find a boundary.
Implementations
impl<'a> Cursor<'a, RopeInfo>
impl<'a> Cursor<'a, RopeInfo>
pub fn prev_codepoint(&mut self) -> Option<char>
pub fn prev_codepoint(&mut self) -> Option<char>
Get previous codepoint before cursor position, and advance cursor backwards.
pub fn next_codepoint(&mut self) -> Option<char>
pub fn next_codepoint(&mut self) -> Option<char>
Get next codepoint after cursor position, and advance cursor.
pub fn peek_next_codepoint(&self) -> Option<char>
pub fn peek_next_codepoint(&self) -> Option<char>
Get the next codepoint after the cursor position, without advancing the cursor.
pub fn next_grapheme(&mut self) -> Option<usize>
pub fn prev_grapheme(&mut self) -> Option<usize>
impl<'a, N> Cursor<'a, N>where
N: NodeInfo,
impl<'a, N> Cursor<'a, N>where
N: NodeInfo,
pub fn new(n: &'a Node<N>, position: usize) -> Cursor<'a, N>
pub fn new(n: &'a Node<N>, position: usize) -> Cursor<'a, N>
Create a new cursor at the given position.
pub fn get_leaf(&self) -> Option<(&'a <N as NodeInfo>::L, usize)>
pub fn get_leaf(&self) -> Option<(&'a <N as NodeInfo>::L, usize)>
Get the current leaf of the cursor.
If the cursor is valid, returns the leaf containing the current position, and the offset of the current position within the leaf. That offset is equal to the leaf length only at the end, otherwise it is less than the leaf length.
pub fn set(&mut self, position: usize)
pub fn set(&mut self, position: usize)
Set the position of the cursor.
The cursor is valid after this call.
Precondition: position is less than or equal to the length of the tree.
pub fn is_boundary<M>(&mut self) -> boolwhere
M: Metric<N>,
pub fn is_boundary<M>(&mut self) -> boolwhere
M: Metric<N>,
Determine whether the current position is a boundary.
Note: the beginning and end of the tree may or may not be boundaries, depending on the
metric. If the metric is not can_fragment, then they always are.
pub fn prev<M>(&mut self) -> Option<usize>where
M: Metric<N>,
pub fn prev<M>(&mut self) -> Option<usize>where
M: Metric<N>,
Moves the cursor to the previous boundary.
When there is no previous boundary, returns None and the cursor becomes invalid.
Return value: the position of the boundary, if it exists.
pub fn next<M>(&mut self) -> Option<usize>where
M: Metric<N>,
pub fn next<M>(&mut self) -> Option<usize>where
M: Metric<N>,
Moves the cursor to the next boundary.
When there is no next boundary, returns None and the cursor becomes invalid.
Return value: the position of the boundary, if it exists.
pub fn at_or_next<M>(&mut self) -> Option<usize>where
M: Metric<N>,
pub fn at_or_next<M>(&mut self) -> Option<usize>where
M: Metric<N>,
pub fn at_or_prev<M>(&mut self) -> Option<usize>where
M: Metric<N>,
pub fn at_or_prev<M>(&mut self) -> Option<usize>where
M: Metric<N>,
pub fn iter<'c, M>(&'c mut self) -> CursorIter<'c, 'a, N, M> ⓘwhere
M: Metric<N>,
pub fn iter<'c, M>(&'c mut self) -> CursorIter<'c, 'a, N, M> ⓘwhere
M: Metric<N>,